package rps;

import static rps.msg.Move.PAPER;
import static rps.msg.Move.ROCK;
import static rps.msg.Move.SCISSORS;

import org.actorsguildframework.Actor;
import org.actorsguildframework.AsyncResult;
import org.actorsguildframework.annotations.Message;
import org.actorsguildframework.annotations.Prop;

import rps.msg.Move;
import rps.msg.ThrowMessage;

public abstract class Coordinator extends Actor {

  @Prop
  public abstract Player getPlayer1();
  public abstract void setPlayer1(Player player);

  @Prop
  public abstract Player getPlayer2();
  public abstract void setPlayer2(Player player);

  @Message
  public AsyncResult<Void> start() {
    AsyncResult<ThrowMessage> result1 = getPlayer1().play();
    AsyncResult<ThrowMessage> result2 = getPlayer2().play();

    ThrowMessage throw1 = result1.get();
    ThrowMessage throw2 = result2.get();

    announce(throw1, throw2);
    
    this.start();
    return noResult();
  }
  
  private void announce(ThrowMessage msg1, ThrowMessage msg2) {
    String winner = "tie";
    Move m1 = msg1.getMove();
    Move m2 = msg2.getMove();
    if(firstWins(m1, m2) && ! firstWins(m2, m1)) {
      winner = msg1.getPlayer();
    } else if(firstWins(m2, m1) && ! firstWins(m1, m2)) {
      winner = msg2.getPlayer();
    } // else tie
    
    if(msg1.getPlayer().compareTo(msg2.getPlayer()) < 0) {
      System.out.println(msg1 + ", " + msg2 + ", winner = " + winner);      
    } else {
      System.out.println(msg2 + ", " + msg1 + ", winner = " + winner);            
    }
  }
  
  private boolean firstWins(Move m1, Move m2) {
    return 
      (m1 == ROCK && m2 == SCISSORS) || 
      (m1 == PAPER && m2 == ROCK) ||
      (m1 == SCISSORS && m2 == PAPER);
  }
}
