package rps;

import static rps.msg.Move.PAPER;
import static rps.msg.Move.ROCK;
import static rps.msg.Move.SCISSORS;
import rps.msg.Message;
import rps.msg.Move;
import rps.msg.PlayMessage;
import rps.msg.StartMessage;
import rps.msg.ThrowMessage;
import kilim.Mailbox;
import kilim.Pausable;
import kilim.Task;

public class Coordinator extends Task {

  private final Mailbox<PlayMessage> player1;
  private final Mailbox<PlayMessage> player2;
  private final Mailbox<Message> coordinator;
  
  private ThrowMessage pendingMessage;

  public Coordinator(Mailbox<Message> coordinator, Mailbox<PlayMessage> player1, Mailbox<PlayMessage> player2) {
    this.player1 = player1;
    this.player2 = player2;
    this.coordinator = coordinator;
  }
  
  public void execute() throws Pausable {
    while(true) {
      Message message = coordinator.get();
      if(message instanceof StartMessage) {
        player1.putnb(new PlayMessage(coordinator));
        player2.putnb(new PlayMessage(coordinator));
      } else {
        ThrowMessage throwMessage = (ThrowMessage) message;
        if(pendingMessage == null) {
          pendingMessage = throwMessage;
        } else {
          announce(pendingMessage, (ThrowMessage) message);
          pendingMessage = null;
          coordinator.putnb(StartMessage.INSTANCE);
        }
      }
    }
  }
  
  public void announce(ThrowMessage msg1, ThrowMessage msg2) {
    String winner = "tie";
    Move m1 = msg1.getMove();
    Move m2 = msg2.getMove();
    if(firstWins(m1, m2) && ! firstWins(m2, m1)) {
      winner = msg1.getPlayer();
    } else if(firstWins(m2, m1) && ! firstWins(m1, m2)) {
      winner = msg2.getPlayer();
    } // else tie
    
    if(msg1.getPlayer().compareTo(msg2.getPlayer()) < 0) {
      System.out.println(msg1 + ", " + msg2 + ", winner = " + winner);      
    } else {
      System.out.println(msg2 + ", " + msg1 + ", winner = " + winner);            
    }
  }
  
  private boolean firstWins(Move m1, Move m2) {
    return 
      (m1 == ROCK && m2 == SCISSORS) || 
      (m1 == PAPER && m2 == ROCK) ||
      (m1 == SCISSORS && m2 == PAPER);
  }

}
