package rps

import scala.actors._
import scala.actors.Actor._
import scala.util.Random

object RockPaperScissors extends Application {
	val c = new Coordinator
	c.start
	c ! Start    
}

abstract case class Move()
case object Rock extends Move
case object Paper extends Move
case object Scissors extends Move

case object Start
case class Play(sender:Actor)
case class Throw(player:Actor, move:Move)

class Coordinator() extends Actor {
  def act() {
    val player1 = new Player().start
	  val player2 = new Player().start

    loop {
      react {
        case Start => {
          player1 ! Play(self)
          player2 ! Play(self)
        }
        case Throw(playerA, throwA) => { 
          react { 
            case Throw(playerB, throwB) => {
              announce(playerA, throwA, playerB, throwB)
              self ! Start
            }
          } 
        }
      }
    }
  }
  
  def announce(playerA:Actor, throwA:Move, playerB:Actor, throwB:Move) {
    val winner = (throwA, throwB) match {
      case (Rock, Paper) => playerB
      case (Rock, Scissors) => playerA
      case (Paper, Rock) => playerA
      case (Paper, Scissors) => playerB
      case (Scissors, Rock) => playerB
      case (Scissors, Paper) => playerA
      case _ => "tie"
    }
    println(playerA + " (" + throwA + "), " + playerB + " (" + throwB + "), winner = " + winner)
  }
}

class Player() extends Actor {
  val random = new Random
  val throws = List(Rock, Paper, Scissors)
  
  def act() {
    loop {
      react {
        case Play(sender) => { 
          val move = throws(random.nextInt(throws.length))
          sender ! Throw(self, move)
        }
      }
    }
  }  
}

